

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

A format definition
-}

module GI.Gst.Structs.FormatDefinition
    ( 

-- * Exported types
    FormatDefinition(..)                    ,
    newZeroFormatDefinition                 ,
    noFormatDefinition                      ,


 -- * Properties
-- ** Description
    formatDefinitionReadDescription         ,


-- ** Nick
    formatDefinitionReadNick                ,


-- ** Quark
    formatDefinitionReadQuark               ,


-- ** Value
    formatDefinitionReadValue               ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gst.Types
import GI.Gst.Callbacks

newtype FormatDefinition = FormatDefinition (ForeignPtr FormatDefinition)
-- | Construct a `FormatDefinition` struct initialized to zero.
newZeroFormatDefinition :: MonadIO m => m FormatDefinition
newZeroFormatDefinition = liftIO $ callocBytes 32 >>= wrapPtr FormatDefinition

noFormatDefinition :: Maybe FormatDefinition
noFormatDefinition = Nothing

formatDefinitionReadValue :: FormatDefinition -> IO Format
formatDefinitionReadValue s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO CUInt
    let val' = (toEnum . fromIntegral) val
    return val'

formatDefinitionReadNick :: FormatDefinition -> IO T.Text
formatDefinitionReadNick s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO CString
    val' <- cstringToText val
    return val'

formatDefinitionReadDescription :: FormatDefinition -> IO T.Text
formatDefinitionReadDescription s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO CString
    val' <- cstringToText val
    return val'

formatDefinitionReadQuark :: FormatDefinition -> IO Word32
formatDefinitionReadQuark s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 24) :: IO Word32
    return val

type family ResolveFormatDefinitionMethod (t :: Symbol) (o :: *) :: * where
    ResolveFormatDefinitionMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveFormatDefinitionMethod t FormatDefinition, MethodInfo info FormatDefinition p) => IsLabelProxy t (FormatDefinition -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveFormatDefinitionMethod t FormatDefinition, MethodInfo info FormatDefinition p) => IsLabel t (FormatDefinition -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


